@@ -7,9 +7,12 @@
 {-# LANGUAGE FlexibleContexts  #-}
 module Yesod.Core.Internal.Run where
 
+
+#if __GLASGOW_HASKELL__ < 710
+import           Data.Monoid                  (mempty)
+#endif
 import Yesod.Core.Internal.Response
 import           Blaze.ByteString.Builder     (toByteString)
-import           Control.Applicative          ((<$>))
 import           Control.Exception            (fromException, evaluate)
 import qualified Control.Exception            as E
 import           Control.Exception.Lifted     (catch)
@@ -25,13 +28,12 @@ import qualified Data.IORef                   as I
 import qualified Data.Map                     as Map
 import           Data.Maybe                   (isJust)
 import           Data.Maybe                   (fromMaybe)
-import           Data.Monoid                  (appEndo, mempty)
+import           Data.Monoid                  (appEndo)
 import           Data.Text                    (Text)
 import qualified Data.Text                    as T
 import           Data.Text.Encoding           (encodeUtf8)
 import           Data.Text.Encoding           (decodeUtf8With)
 import           Data.Text.Encoding.Error     (lenientDecode)
-import           Data.Time                    (getCurrentTime, addUTCTime)
 import           Language.Haskell.TH.Syntax   (Loc, qLocation)
 import qualified Network.HTTP.Types           as H
 import           Network.Wai
@@ -40,17 +42,14 @@ import           Network.Wai.Internal
 import           Prelude                      hiding (catch)
 #endif
 import           System.Log.FastLogger        (LogStr, toLogStr)
-import           System.Random                (newStdGen)
 import           Yesod.Core.Content
 import           Yesod.Core.Class.Yesod
 import           Yesod.Core.Types
 import           Yesod.Core.Internal.Request  (parseWaiRequest,
                                                tooLargeResponse)
-import           Yesod.Core.Internal.Util     (formatRFC1123)
+import           Yesod.Core.Internal.Util     (getCurrentMaxExpiresRFC1123)
 import           Yesod.Routes.Class           (Route, renderRoute)
-import Control.DeepSeq (($!!), NFData)
-import Control.Monad (liftM)
-import Control.AutoUpdate (mkAutoUpdate, defaultUpdateSettings, updateAction, updateFreq)
+import           Control.DeepSeq              (($!!))
 
 returnDeepSessionMap :: Monad m => SessionMap -> m SessionMap
 #if MIN_VERSION_bytestring(0, 10, 0)
@@ -197,7 +196,7 @@ runFakeHandler :: (Yesod site, MonadIO m) =>
                -> m (Either ErrorResponse a)
 runFakeHandler fakeSessionMap logger site handler = liftIO $ do
   ret <- I.newIORef (Left $ InternalError "runFakeHandler: no result")
-  getMaxExpires <- getGetMaxExpires
+  maxExpires <- getCurrentMaxExpiresRFC1123
   let handler' = do liftIO . I.writeIORef ret . Right =<< handler
                     return ()
   let yapp = runHandler
@@ -208,7 +207,7 @@ runFakeHandler fakeSessionMap logger site handler = liftIO $ do
             , rheUpload = fileUpload site
             , rheLog = messageLoggerSource site $ logger site
             , rheOnError = errHandler
-            , rheGetMaxExpires = getMaxExpires
+            , rheMaxExpires = maxExpires
             }
         handler'
       errHandler err req = do
@@ -259,11 +258,11 @@ yesodRunner handler' YesodRunnerEnv {..} route req sendResponse
     let dontSaveSession _ = return []
     (session, saveSession) <- liftIO $ do
         maybe (return (Map.empty, dontSaveSession)) (\sb -> sbLoadSession sb req) yreSessionBackend
-    getMaxExpires <- getGetMaxExpires
+    maxExpires <- yreGetMaxExpires
     let mkYesodReq = parseWaiRequest req session (isJust yreSessionBackend) mmaxLen
     let yreq =
             case mkYesodReq of
-                Left yreq -> yreq
+                Left yreq' -> yreq'
                 Right needGen -> needGen yreGen
     let ra = resolveApproot yreSite req
     let log' = messageLoggerSource yreSite yreLogger
@@ -278,7 +277,7 @@ yesodRunner handler' YesodRunnerEnv {..} route req sendResponse
             , rheUpload = fileUpload yreSite
             , rheLog = log'
             , rheOnError = safeEh log'
-            , rheGetMaxExpires = getMaxExpires
+            , rheMaxExpires = maxExpires
             }
         rhe = rheSafe
             { rheOnError = runHandler rheSafe . errorHandler
@@ -292,12 +291,6 @@ yesodRunner handler' YesodRunnerEnv {..} route req sendResponse
     mmaxLen = maximumContentLength yreSite route
     handler = yesodMiddleware handler'
 
-getGetMaxExpires :: MonadIO m => m (IO Text)
-getGetMaxExpires = liftIO $ mkAutoUpdate defaultUpdateSettings
-  { updateAction = liftM (formatRFC1123 . addUTCTime (60*60*24*365)) getCurrentTime
-  , updateFreq = 60 * 60 * 1000000 -- Update once per hour
-  }
-
 yesodRender :: Yesod y
             => y
             -> ResolvedApproot
