@@ -29,7 +29,6 @@ module Yesod.Core.Json
 
 import Yesod.Core.Handler (HandlerT, getRequest, invalidArgs, redirect, selectRep, provideRep, rawRequestBody, ProvidedRep)
 import Control.Monad.Trans.Writer (Writer)
-import Control.Monad.Trans.Resource (runExceptionT)
 import Data.Monoid (Endo)
 import Yesod.Core.Content (TypedContent)
 import Yesod.Core.Types (reqAccept)
@@ -95,11 +94,7 @@ provideJson = provideRep . return . J.toJSON
 -- /Since: 0.3.0/
 parseJsonBody :: (MonadHandler m, J.FromJSON a) => m (J.Result a)
 parseJsonBody = do
-#if MIN_VERSION_resourcet(1,1,0)
     eValue <- rawRequestBody $$ runCatchC (sinkParser JP.value')
-#else
-    eValue <- runExceptionT $ rawRequestBody $$ sinkParser JP.value'
-#endif
     return $ case eValue of
         Left e -> J.Error $ show e
         Right value -> J.fromJSON value
