@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE DeriveDataTypeable         #-}
 {-# LANGUAGE TupleSections #-}
 {-# LANGUAGE FlexibleInstances          #-}
@@ -5,38 +6,33 @@
 {-# LANGUAGE MultiParamTypeClasses      #-}
 {-# LANGUAGE TypeFamilies               #-}
 {-# LANGUAGE UndecidableInstances #-}
-{-# LANGUAGE CPP #-}
 module Yesod.Core.Types where
 
 import qualified Blaze.ByteString.Builder           as BBuilder
 import qualified Blaze.ByteString.Builder.Char.Utf8
+#if __GLASGOW_HASKELL__ < 710
 import           Control.Applicative                (Applicative (..))
 import           Control.Applicative                ((<$>))
+import           Data.Monoid                        (Monoid (..))
+#endif
 import           Control.Arrow                      (first)
 import           Control.Exception                  (Exception)
 import           Control.Monad                      (liftM, ap)
 import           Control.Monad.Base                 (MonadBase (liftBase))
 import           Control.Monad.Catch                (MonadCatch (..))
-#if MIN_VERSION_exceptions(0,6,0)
 import           Control.Monad.Catch                (MonadMask (..))
-#endif
 import           Control.Monad.IO.Class             (MonadIO (liftIO))
 import           Control.Monad.Logger               (LogLevel, LogSource,
                                                      MonadLogger (..))
 import           Control.Monad.Trans.Control        (MonadBaseControl (..))
 import           Control.Monad.Trans.Resource       (MonadResource (..), InternalState, runInternalState, MonadThrow (..), monadThrow, ResourceT)
-#if !MIN_VERSION_resourcet(1,1,0)
-import           Control.Monad.Trans.Resource       (MonadUnsafeIO (..))
-#endif
 import           Data.ByteString                    (ByteString)
 import qualified Data.ByteString.Lazy               as L
 import           Data.Conduit                       (Flush, Source)
-import           Data.Dynamic                       (Dynamic)
 import           Data.IORef                         (IORef)
 import           Data.Map                           (Map, unionWith)
 import qualified Data.Map                           as Map
-import           Data.Monoid                        (Endo (..), Last (..),
-                                                     Monoid (..))
+import           Data.Monoid                        (Endo (..), Last (..))
 import           Data.Serialize                     (Serialize (..),
                                                      putByteString)
 import           Data.String                        (IsString (fromString))
@@ -45,20 +41,16 @@ import qualified Data.Text                          as T
 import qualified Data.Text.Lazy.Builder             as TBuilder
 import           Data.Time                          (UTCTime)
 import           Data.Typeable                      (Typeable)
-import           Data.Typeable                      (TypeRep)
 import           Language.Haskell.TH.Syntax         (Loc)
 import qualified Network.HTTP.Types                 as H
 import           Network.Wai                        (FilePart,
                                                      RequestBodyLength)
 import qualified Network.Wai                        as W
 import qualified Network.Wai.Parse                  as NWP
-#if MIN_VERSION_fast_logger(2, 0, 0)
 import           System.Log.FastLogger              (LogStr, LoggerSet, toLogStr, pushLogStr)
+import qualified System.Random.MWC                  as MWC
 import           Network.Wai.Logger                 (DateCacheGetter)
-#else
-import           System.Log.FastLogger              (LogStr, Logger, toLogStr)
-#endif
-import           Text.Blaze.Html                    (Html)
+import           Text.Blaze.Html                    (Html, toHtml)
 import           Text.Hamlet                        (HtmlUrl)
 import           Text.Julius                        (JavascriptUrl)
 import           Web.Cookie                         (SetCookie)
@@ -66,11 +58,16 @@ import           Yesod.Core.Internal.Util           (getTime, putTime)
 import           Control.Monad.Trans.Class          (MonadTrans (..))
 import           Yesod.Routes.Class                 (RenderRoute (..), ParseRoute (..))
 import           Control.Monad.Reader               (MonadReader (..))
+#if !MIN_VERSION_base(4, 6, 0)
 import Prelude hiding (catch)
-import Control.DeepSeq (NFData (rnf))
-#if MIN_VERSION_conduit(1, 1, 0)
-import Data.Conduit.Lazy (MonadActive, monadActive)
 #endif
+import Control.DeepSeq (NFData (rnf))
+import Data.Conduit.Lazy (MonadActive, monadActive)
+import Yesod.Core.TypeCache (TypeMap, KeyedTypeMap)
+#if MIN_VERSION_monad_logger(0, 3, 10)
+import Control.Monad.Logger (MonadLoggerIO (..))
+#endif
+import Data.Semigroup (Semigroup)
 
 -- Sessions
 type SessionMap = Map Text ByteString
@@ -146,11 +143,7 @@ data FileInfo = FileInfo
     }
 
 data FileUpload = FileUploadMemory !(NWP.BackEnd L.ByteString)
-#if MIN_VERSION_wai_extra(2, 0, 1)
                 | FileUploadDisk !(InternalState -> NWP.BackEnd FilePath)
-#else
-                | FileUploadDisk !(NWP.BackEnd FilePath)
-#endif
                 | FileUploadSource !(NWP.BackEnd (Source (ResourceT IO) ByteString))
 
 -- | How to determine the root of the application for constructing URLs.
@@ -178,9 +171,6 @@ type BottomOfHeadAsync master
       -> Maybe (HtmlUrl (Route master)) -- ^ widget of js to run on async completion
       -> (HtmlUrl (Route master)) -- ^ widget to insert at the bottom of <head>
 
-newtype Cache = Cache (Map TypeRep Dynamic)
-    deriving Monoid
-
 type Texts = [Text]
 
 -- | Wrap up a normal WAI application as a Yesod subsite.
@@ -196,6 +186,7 @@ data RunHandlerEnv site = RunHandlerEnv
       -- ^ How to respond when an error is thrown internally.
       --
       -- Since 1.2.0
+    , rheMaxExpires :: !Text
     }
 
 data HandlerData site parentRoute = HandlerData
@@ -210,6 +201,8 @@ data YesodRunnerEnv site = YesodRunnerEnv
     { yreLogger         :: !Logger
     , yreSite           :: !site
     , yreSessionBackend :: !(Maybe SessionBackend)
+    , yreGen            :: !MWC.GenIO
+    , yreGetMaxExpires  :: IO Text
     }
 
 data YesodSubRunnerEnv sub parent parentMonad = YesodSubRunnerEnv
@@ -239,7 +232,8 @@ data GHState = GHState
     { ghsSession :: SessionMap
     , ghsRBC     :: Maybe RequestBodyContents
     , ghsIdent   :: Int
-    , ghsCache   :: Cache
+    , ghsCache   :: TypeMap
+    , ghsCacheBy :: KeyedTypeMap
     , ghsHeaders :: Endo [Header]
     }
 
@@ -258,6 +252,17 @@ newtype WidgetT site m a = WidgetT
 instance (a ~ (), Monad m) => Monoid (WidgetT site m a) where
     mempty = return ()
     mappend x y = x >> y
+instance (a ~ (), Monad m) => Semigroup (WidgetT site m a)
+
+-- | A 'String' can be trivially promoted to a widget.
+--
+-- For example, in a yesod-scaffold site you could use:
+--
+-- @getHomeR = do defaultLayout "Widget text"@
+instance (Monad m, a ~ ()) => IsString (WidgetT site m a) where
+    fromString = toWidget . toHtml . T.pack
+      where toWidget x = WidgetT $ const $ return $ ((), GWData (Body (const x))
+                         mempty mempty mempty mempty mempty mempty)
 
 type RY master = Route master -> [(Text, Text)] -> Text
 
@@ -301,8 +306,7 @@ type ContentType = ByteString -- FIXME Text?
 -- Since 1.1.0
 newtype DontFullyEvaluate a = DontFullyEvaluate { unDontFullyEvaluate :: a }
 
--- | Responses to indicate some form of an error occurred. These are different
--- from 'SpecialResponse' in that they allow for custom error pages.
+-- | Responses to indicate some form of an error occurred.
 data ErrorResponse =
       NotFound
     | InternalError Text
@@ -343,8 +347,10 @@ newtype Title = Title { unTitle :: Html }
 
 newtype Head url = Head (HtmlUrl url)
     deriving Monoid
+instance Semigroup (Head a)
 newtype Body url = Body (HtmlUrl url)
     deriving Monoid
+instance Semigroup (Body a)
 
 type CssBuilderUrl a = (a -> [(Text, Text)] -> Text) -> TBuilder.Builder
 
@@ -368,6 +374,7 @@ instance Monoid (GWData a) where
         (unionWith mappend a5 b5)
         (a6 `mappend` b6)
         (a7 `mappend` b7)
+instance Semigroup (GWData a)
 
 data HandlerContents =
       HCContent H.Status !TypedContent
@@ -406,12 +413,21 @@ instance MonadIO m => MonadIO (WidgetT site m) where
 instance MonadBase b m => MonadBase b (WidgetT site m) where
     liftBase = WidgetT . const . liftBase . fmap (, mempty)
 instance MonadBaseControl b m => MonadBaseControl b (WidgetT site m) where
+#if MIN_VERSION_monad_control(1,0,0)
+    type StM (WidgetT site m) a = StM m (a, GWData (Route site))
+    liftBaseWith f = WidgetT $ \reader' ->
+        liftBaseWith $ \runInBase ->
+            liftM (\x -> (x, mempty))
+            (f $ runInBase . flip unWidgetT reader')
+    restoreM = WidgetT . const . restoreM
+#else
     data StM (WidgetT site m) a = StW (StM m (a, GWData (Route site)))
     liftBaseWith f = WidgetT $ \reader' ->
         liftBaseWith $ \runInBase ->
             liftM (\x -> (x, mempty))
             (f $ liftM StW . runInBase . flip unWidgetT reader')
     restoreM (StW base) = WidgetT $ const $ restoreM base
+#endif
 instance Monad m => MonadReader site (WidgetT site m) where
     ask = WidgetT $ \hd -> return (rheSite $ handlerEnv hd, mempty)
     local f (WidgetT g) = WidgetT $ \hd -> g hd
@@ -423,14 +439,11 @@ instance Monad m => MonadReader site (WidgetT site m) where
 instance MonadTrans (WidgetT site) where
     lift = WidgetT . const . liftM (, mempty)
 instance MonadThrow m => MonadThrow (WidgetT site m) where
-#if MIN_VERSION_resourcet(1,1,0)
     throwM = lift . throwM
 
 instance MonadCatch m => MonadCatch (HandlerT site m) where
   catch (HandlerT m) c = HandlerT $ \r -> m r `catch` \e -> unHandlerT (c e) r
-#if MIN_VERSION_exceptions(0,6,0)
 instance MonadMask m => MonadMask (HandlerT site m) where
-#endif
   mask a = HandlerT $ \e -> mask $ \u -> unHandlerT (a $ q u) e
     where q u (HandlerT b) = HandlerT (u . b)
   uninterruptibleMask a =
@@ -438,35 +451,29 @@ instance MonadMask m => MonadMask (HandlerT site m) where
       where q u (HandlerT b) = HandlerT (u . b)
 instance MonadCatch m => MonadCatch (WidgetT site m) where
   catch (WidgetT m) c = WidgetT $ \r -> m r `catch` \e -> unWidgetT (c e) r
-#if MIN_VERSION_exceptions(0,6,0)
 instance MonadMask m => MonadMask (WidgetT site m) where
-#endif
   mask a = WidgetT $ \e -> mask $ \u -> unWidgetT (a $ q u) e
     where q u (WidgetT b) = WidgetT (u . b)
   uninterruptibleMask a =
     WidgetT $ \e -> uninterruptibleMask $ \u -> unWidgetT (a $ q u) e
       where q u (WidgetT b) = WidgetT (u . b)
-#else
-    monadThrow = lift . monadThrow
-#endif
 
-#if MIN_VERSION_resourcet(1,1,0)
 instance (Applicative m, MonadIO m, MonadBase IO m, MonadThrow m) => MonadResource (WidgetT site m) where
-#else
-instance (Applicative m, MonadIO m, MonadUnsafeIO m, MonadThrow m) => MonadResource (WidgetT site m) where
-#endif
     liftResourceT f = WidgetT $ \hd -> liftIO $ fmap (, mempty) $ runInternalState f (handlerResource hd)
 
 instance MonadIO m => MonadLogger (WidgetT site m) where
     monadLoggerLog a b c d = WidgetT $ \hd ->
         liftIO $ fmap (, mempty) $ rheLog (handlerEnv hd) a b c (toLogStr d)
 
-#if MIN_VERSION_conduit(1, 1, 0)
+#if MIN_VERSION_monad_logger(0, 3, 10)
+instance MonadIO m => MonadLoggerIO (WidgetT site m) where
+    askLoggerIO = WidgetT $ \hd -> return (rheLog (handlerEnv hd), mempty)
+#endif
+
 instance MonadActive m => MonadActive (WidgetT site m) where
     monadActive = lift monadActive
 instance MonadActive m => MonadActive (HandlerT site m) where
     monadActive = lift monadActive
-#endif
 
 instance MonadTrans (HandlerT site) where
     lift = HandlerT . const
@@ -500,33 +507,39 @@ instance Monad m => MonadReader site (HandlerT site m) where
 -- \"Control.Monad.Trans.Resource.register\': The mutable state is being accessed
 -- after cleanup. Please contact the maintainers.\"
 instance MonadBaseControl b m => MonadBaseControl b (HandlerT site m) where
+#if MIN_VERSION_monad_control(1,0,0)
+    type StM (HandlerT site m) a = StM m a
+    liftBaseWith f = HandlerT $ \reader' ->
+        liftBaseWith $ \runInBase ->
+            f $ runInBase . (\(HandlerT r) -> r reader')
+    restoreM = HandlerT . const . restoreM
+#else
     data StM (HandlerT site m) a = StH (StM m a)
     liftBaseWith f = HandlerT $ \reader' ->
         liftBaseWith $ \runInBase ->
             f $ liftM StH . runInBase . (\(HandlerT r) -> r reader')
     restoreM (StH base) = HandlerT $ const $ restoreM base
+#endif
 
 instance MonadThrow m => MonadThrow (HandlerT site m) where
-#if MIN_VERSION_resourcet(1,1,0)
     throwM = lift . monadThrow
-#else
-    monadThrow = lift . monadThrow
-#endif
 
-#if MIN_VERSION_resourcet(1,1,0)
 instance (MonadIO m, MonadBase IO m, MonadThrow m) => MonadResource (HandlerT site m) where
-#else
-instance (MonadIO m, MonadUnsafeIO m, MonadThrow m) => MonadResource (HandlerT site m) where
-#endif
     liftResourceT f = HandlerT $ \hd -> liftIO $ runInternalState f (handlerResource hd)
 
 instance MonadIO m => MonadLogger (HandlerT site m) where
     monadLoggerLog a b c d = HandlerT $ \hd ->
         liftIO $ rheLog (handlerEnv hd) a b c (toLogStr d)
 
+#if MIN_VERSION_monad_logger(0, 3, 10)
+instance MonadIO m => MonadLoggerIO (HandlerT site m) where
+    askLoggerIO = HandlerT $ \hd -> return (rheLog (handlerEnv hd))
+#endif
+
 instance Monoid (UniqueList x) where
     mempty = UniqueList id
     UniqueList x `mappend` UniqueList y = UniqueList $ x . y
+instance Semigroup (UniqueList x)
 
 instance IsString Content where
     fromString = flip ContentBuilder Nothing . Blaze.ByteString.Builder.Char.Utf8.fromString
@@ -538,7 +551,6 @@ instance RenderRoute WaiSubsite where
 instance ParseRoute WaiSubsite where
     parseRoute (x, y) = Just $ WaiSubsiteRoute x y
 
-#if MIN_VERSION_fast_logger(2, 0, 0)
 data Logger = Logger
     { loggerSet :: !LoggerSet
     , loggerDate :: !DateCacheGetter
@@ -546,4 +558,3 @@ data Logger = Logger
 
 loggerPutStr :: Logger -> LogStr -> IO ()
 loggerPutStr (Logger ls _) = pushLogStr ls
-#endif
