@@ -15,13 +15,13 @@ import qualified YesodCoreTest.Redirect as Redirect
 import qualified YesodCoreTest.JsLoader as JsLoader
 import qualified YesodCoreTest.RequestBodySize as RequestBodySize
 import qualified YesodCoreTest.Json as Json
-#if MIN_VERSION_wai(2, 1, 0)
 import qualified YesodCoreTest.RawResponse as RawResponse
-#endif
 import qualified YesodCoreTest.Streaming as Streaming
 import qualified YesodCoreTest.Reps as Reps
 import qualified YesodCoreTest.Auth as Auth
 import qualified YesodCoreTest.LiteApp as LiteApp
+import qualified YesodCoreTest.Ssl as Ssl
+import qualified YesodCoreTest.Csrf as Csrf
 
 import Test.Hspec
 
@@ -41,10 +41,11 @@ specs = do
       JsLoader.specs
       RequestBodySize.specs
       Json.specs
-#if MIN_VERSION_wai(2, 1, 0)
       RawResponse.specs
-#endif
       Streaming.specs
       Reps.specs
       Auth.specs
       LiteApp.specs
+      Ssl.unsecSpec
+      Ssl.sslOnlySpec
+      Csrf.csrfSpec
